use ndarray::{Array3, Ix2, iter::LanesMut};

use crate::{geometry::Transform2, sampling::Sampler};

pub struct Film {
    pub width: u32,
    pub height: u32,
    pub transform: Transform2,
    pub data: Array3<f32>,
}

impl Film {
    pub fn new(width: u32, height: u32) -> Self {
        let w = width as f32;
        let h = height as f32;
        let ar = h / w;

        let transform = Transform2::translate(-1.0, ar) * Transform2::scale(2.0 / w, -2.0 / w);

        let data = Array3::zeros((height as usize, width as usize, 3));

        Self {
            width,
            height,
            transform,
            data,
        }
    }

    pub fn sampler(&self, samples: u32, seed: u32) -> Sampler {
        Sampler::new(self.width, self.height, samples, seed)
    }

    pub fn lanes_mut(&mut self) -> LanesMut<'_, f32, Ix2> {
        self.data.lanes_mut(ndarray::Axis(2))
    }
}
